import re
import papis.downloaders.base


class Downloader(papis.downloaders.base.Downloader):

    def __init__(self, url):
        papis.downloaders.base.Downloader.__init__(self, url)

    @classmethod
    def match(cls, url):
        """
        >>> Downloader.match(\
                'blah://pubs.acs.org/doi/abs/10.1021/acs.jchemed.6b00559'\
            ) is False
        False
        >>> Downloader.match(\
                'blah://pubs.acs.org/!@#!@$!%!@%!$che.6b00559'\
            ) is False
        False
        >>> Downloader.match(\
                'acs.com/!@#!@$!%!@%!$chemed.6b00559'\
            ) is False
        True
        """
        if re.match(r".*acs.org.*", url):
            return Downloader(url)
        else:
            return False

    def getDoi(self):
        # http://pubs.acs.org/doi/whatever/10.1021/acs.jchemed.6b00559
        mdoi = re.match(r'.*acs.org/doi/[^/]+/(.*)', self.getUrl())
        if mdoi:
            doi = mdoi.group(1).replace("abs/", "").replace("full/", "")
            self.logger.debug("[doi] = %s" % doi)
            return doi
        else:
            self.logger.error("Doi not found!!")

    def getDocumentUrl(self):
        # http://pubs.acs.org/doi/pdf/10.1021/acs.jchemed.6b00559
        durl = "http://pubs.acs.org/doi/pdf/" + self.getDoi()
        self.logger.debug("[doc url] = %s" % durl)
        self.logger.error("Retrieving urls does not work for acs yet")
        raise Exception
        return durl

    def getBibtexUrl(self):
        url = "http://pubs.acs.org/action/downloadCitation"\
              "?format=bibtex&cookieSet=1&doi=%s" % self.getDoi()
        self.logger.debug("[bibtex url] = %s" % url)
        return url

# vim-run: python3 -m doctest %



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import logging


logger = logging.getLogger("papis")
if "--debug" in sys.argv:
    log_format = '%(relativeCreated)d-'
    log_format += '%(levelname)s:%(name)s:%(message)s'
    logging.basicConfig(level=logging.DEBUG, format=log_format)
    sys.argv.pop(sys.argv.index("--debug"))
    logger.debug("DEBUG MODE FOR DEVELOPERS ON")


import papis.commands
import papis.api
logger.debug("Imported commands")


def main():
    try:
        papis.commands.main()
    except KeyboardInterrupt:
        print('Getting you out of here...')


if sys.version_info < (3, 2):
    raise Exception("This program must use python 3.2 or greater")

if __name__ == "__main__":
    main()

# vim:set et sw=4 ts=4 ft=python:



"""
This command is to list contents of a library.
"""

import papis
import os
import sys
import papis.utils
import papis.downloaders.utils


class Command(papis.commands.Command):
    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "list",
            help="List documents from a given library"
        )

        self.add_search_argument()

        self.parser.add_argument(
            "-i",
            "--info",
            help="Show the info file name associated with the document",
            default=False,
            action="store_true"
        )

        self.parser.add_argument(
            "-f",
            "--file",
            help="Show the file name associated with the document",
            action="store_true"
        )

        self.parser.add_argument(
            "-d",
            "--dir",
            help="Show the folder name associated with the document",
            action="store_true"
        )

        self.parser.add_argument(
            "--format",
            help="List entries using special format",
            default=None,
            action="store"
        )

        self.parser.add_argument(
            "--template",
            help="Use template file for formating output",
            default=None,
            action="store"
        )

        self.parser.add_argument(
            "-p",
            "--pick",
            help="Pick the document instead of listing everything",
            action="store_true"
        )

        self.parser.add_argument(
            "--downloaders",
            help="List available downloaders",
            action="store_true"
        )

    def main(self):
        if self.args.template:
            if not os.path.exists(self.args.template):
                self.logger.error(
                    "Template file %s not found" % self.args.template
                )
                sys.exit(1)
            fd = open(self.args.template)
            self.args.format = fd.read()
            fd.close()
        if self.args.downloaders:
            for downloader in \
               papis.downloaders.utils.getAvailableDownloaders():
                print(downloader)
            sys.exit(0)

        documents = papis.api.get_documents_in_lib(
            self.get_args().lib,
            self.get_args().search
        )

        if self.args.pick:
            documents = [self.pick(documents)]
        for document in documents:
            if self.args.file:
                for f in document.get_files():
                    print(f)
            elif self.args.info:
                print(
                    os.path.join(
                        document.get_main_folder(),
                        document.get_info_file()
                    )
                )
            elif self.args.format:
                print(
                    papis.utils.format_doc(self.args.format, document)
                )
            else:
                print(document.get_main_folder())



import logging

logger = logging.getLogger("commands")
logger.debug("importing")

import os
import papis.utils
import papis.config
COMMAND_NAMES = [
    "default",
    "add",
    "check",
    "config",
    "edit",
    "export",
    "explore",
    "list",
    "rm",
    "mv",
    "rename",
    "open",
    "browse",
    "update",
    "run",
    "git",
    "gui",
    "sync"
]

DEFAULT_PARSER = None
SUBPARSERS = None
COMMANDS = None
ARGS = None


def set_args(args):
    """
    Set general command line arguments, this can be used also for testing.

    :param args: Arguments
    :type  args: Argument object
    """
    global ARGS
    global logger
    logger.debug("Setting args")
    if ARGS is None:
        ARGS = args


def set_commands(commands):
    """
    Set general initialized commands.

    :param commands: List of initialized command objects
    :type  commands: List
    """
    global COMMANDS
    logger.debug("Setting commands")
    COMMANDS = commands


def get_commands(command=None):
    """
    Get general initialized commands.

    :param command: Command that should be returned.
    :type  command: str
    >>> get_commands() is not None
    True
    >>> type(get_commands()) is dict
    True
    >>> 'add' in get_commands().keys()
    True
    """
    global COMMANDS
    if COMMANDS is None:
        init_commands()
    if command is None:
        return COMMANDS
    else:
        return COMMANDS[command]


def list_commands():
    """List all available commands
    :returns: List containing the names of the commands
    :rtype:  list

    >>> len(list_commands()) > 0
    True
    >>> type(list_commands()) is list
    True
    >>> 'add' in list_commands() and 'open' in list_commands()
    True
    >>> 'default' in list_commands()
    False
    >>> 'external' in list_commands()
    False
    """
    return [
        cmd for cmd in get_commands().keys()
        if cmd not in ['default', 'external']
    ]


def get_args():
    """
    Get general command line arguments.
    """
    global ARGS
    return ARGS


def get_default_parser():
    import argparse
    global DEFAULT_PARSER
    global logger
    if DEFAULT_PARSER is None:
        DEFAULT_PARSER = argparse.ArgumentParser(
            formatter_class=argparse.RawTextHelpFormatter,
            description="Simple documents administration program"
        )
    return DEFAULT_PARSER


def get_subparsers():
    global SUBPARSERS
    global logger
    if SUBPARSERS is None:
        SUBPARSER_HELP = "For further information for every "\
                         "command, type in 'papis <command> -h'"
        SUBPARSERS = get_default_parser().add_subparsers(
            help=SUBPARSER_HELP,
            metavar="command",
            dest="command"
        )
    return SUBPARSERS


def get_command_class_by_name(name):
    """This returns returns a command class ready to be initialised

    :param name: Name of the command, e.g., add
    :type  name: str
    :returns: A command not initialized
    :rtype: papis.commands.Command
    """
    exec("import papis.commands.%s" % (name))
    return eval("papis.commands.%s.Command" % name)


def init_internal_commands():
    global COMMAND_NAMES
    global logger
    commands = dict()
    cmd = None
    logger.debug("Initializing internal commands")
    for command in COMMAND_NAMES:
        logger.debug(command)
        cmd = get_command_class_by_name(command)()
        cmd.init()
        commands[command] = cmd
    return commands


def init_external_commands():
    import glob
    from papis.commands.external import Command as External
    logger.debug("Initializing external commands")
    commands = dict()
    paths = []
    paths.append(papis.config.get_scripts_folder())
    paths += os.environ["PATH"].split(":")
    for path in paths:
        scripts = glob.glob(os.path.join(path, "papis-*"))
        if len(scripts):
            for script in scripts:
                cmd = External()
                logger.debug(script)
                cmd.init(script)
                commands[cmd.get_command_name()] = cmd
    logger.debug("Initializing external commands done")
    return commands


def init_commands():
    """Initialize all the commands
    """
    commands = dict()
    commands.update(init_internal_commands())
    commands.update(init_external_commands())
    set_commands(commands)


def init():
    import argcomplete
    if get_commands() is not None:
        raise RuntimeError("Commands are already initialised")
    init_commands()
    # autocompletion
    argcomplete.autocomplete(get_default_parser())
    return get_commands()


def main(input_args=[]):
    commands = get_commands()
    # Parse arguments
    args = get_default_parser().parse_args(input_args or None)
    set_args(args)
    logger.debug("running main")
    commands["default"].main()


def init_and_return_parser():
    """This function is here for the automatic documentation of the
    subcommands.
    :returns: General command line parser
    """
    try:
        init()
    except Exception:
        pass
    finally:
        return get_default_parser()


class Command(object):

    parser = None
    args = None

    def __init__(self):
        self.default_parser = get_default_parser()
        self.subparsers = get_subparsers()
        self.logger = logging.getLogger(self.__class__.__name__)

    def init(self):
        pass

    def main(self):
        pass

    def add_search_argument(self):
        self.parser.add_argument(
            "search",
            help="Search query string",
            nargs="?",
            default=".",
            action="store"
        )

    def add_git_argument(
            self,
            flags=['--git'],
            help="Run command involving git",
            action=None
            ):
        action = action or ('store_false' if papis.config.get('use-git')
                                else 'store_true')
        self.parser.add_argument(
            *flags,
            help=help,
            action=action
        )

    def set_args(self, args):
        self.args = args

    def set_parser(self, parser):
        self.parser = parser

    def set_subparsers(self, subparsers):
        self.subparsers = subparsers

    def get_config(self):
        """Get configuration for the whole papis. It just simply retrieves the
        general configuration using the main ``papis.config`` API.

        :returns: General configuration
        :rtype: dict
        """
        return papis.config.get_configuration()

    def get_parser(self):
        return self.parser

    def get_args(self):
        return self.args

    def get_subparsers(self):
        return self.subparsers

    def pick(self, options, pick_config={}):
        import papis.api
        self.logger.debug("Picking")
        if len(options) == 0:
            return None
        if not pick_config:
            header_format = papis.config.get("header-format")
            match_format = papis.config.get("match-format")
            pick_config = dict(
                header_filter=lambda x: papis.utils.format_doc(header_format, x),
                match_filter=lambda x: papis.utils.format_doc(match_format, x)
            )
        return papis.api.pick(
            options,
            pick_config
        )



import logging

logger = logging.getLogger("config")
logger.debug("importing")

import os
import configparser
import papis.exceptions
import papis.utils


CONFIGURATION = None #: Global configuration object variable.
DEFAULT_SETTINGS = None #: Default settings for the whole papis.
DEFAULT_MODE = "document" #: Default mode in the modal architecture.
OVERRIDE_VARS = {
    "folder": None,
    "cache": None,
    "file": None,
    "scripts": None
}


general_settings = {
    "mode"            : "document",
    "opentool"        : "xdg-open",
    "browser"         : "xdg-open",
    "picktool"        : "papis.pick",
    "mvtool"          : "mv",
    "editor"          : "xdg-open",
    "xeditor"         : "xdg-open",
    "sync-command"    : "git -C $dir pull origin master",
    "notes-name"      : "notes.tex",
    "format-doc-name" : "doc",
    "use-cache"       : True,
    "use-git"       : False,
    "add-confirm"     : False,
    "add-interactive" : False,
    "add-edit"        : False,
    "add-open"        : False,
    "check-keys"      : 'files',
    "browse-query-format"   : "{doc[title]} {doc[author]}",
    "search-engine"   : "https://duckduckgo.com",
    "user-agent"      : \
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) '
        'AppleWebKit/537.36 (KHTML, like Gecko) '
        'Chrome/35.0.1916.47 Safari/537.36',
    "default-gui"     : "vim",
    "scripts-short-help-regex": ".*papis-short-help: *(.*)",
    "info-name"       : "info.yaml",
    "doc-url-key-name": "doc_url",
    "file-browser"    : "xdg-open",
    "default-library" : "papers",
    "export-text-format" : \
        "{doc[author]}. {doc[title]}. {doc[journal]} {doc[pages]}"
        " {doc[month]} {doc[year]}",
    "match-format"    : \
        "{doc[tags]}{doc.subfolder}{doc[title]}{doc[author]}{doc[year]}",
    "header-format"   : \
        "{doc[title]:<70.70}|{doc[author]:<20.20} ({doc[year]:-<4})",
}


def get_general_settings_name():
    """Get the section name of the general settings
    :returns: Section's name
    :rtype:  str
    >>> get_general_settings_name()
    'settings'
    """
    return "settings"


def get_default_settings(section="", key=""):
    """Get the default settings for all non-user variables
    in papis.

    If section and key are given, then the setting
    for the given section and the given key are returned.

    If only ``key`` is given, then the setting
    for the ``general`` section is returned.

    :param section: Particular section of the default settings
    :type  section: str
    :param key: Setting's name to be queried for.
    :type  key: str

    >>> import collections
    >>> type(get_default_settings()) is collections.OrderedDict
    True
    >>> get_default_settings(key='mvtool')
    'mv'
    >>> get_default_settings(key='help-key', section='vim-gui')
    'h'
    """
    global DEFAULT_SETTINGS
    import papis.gui
    # We use an OrderedDict so that the first entry will always be the general
    # settings, also good for automatic documentation
    from collections import OrderedDict
    if DEFAULT_SETTINGS is None:
        DEFAULT_SETTINGS = OrderedDict()
        DEFAULT_SETTINGS.update({
            get_general_settings_name(): general_settings,
        })
        DEFAULT_SETTINGS.update(
            papis.gui.get_default_settings()
        )
    if not section and not key:
        return DEFAULT_SETTINGS
    elif not section:
        return DEFAULT_SETTINGS[get_general_settings_name()][key]
    else:
        return DEFAULT_SETTINGS[section][key]


def get_config_folder():
    """Get folder where the configuration files are stored,
    e.g. /home/user/.papis
    """
    return os.path.join(
        os.path.expanduser("~"), ".papis"
    )


def get_config_file():
    """Get the path of the main configuration file,
    e.g. /home/user/.papis/config
    """
    global OVERRIDE_VARS
    if OVERRIDE_VARS["file"] is not None:
        config_file = OVERRIDE_VARS["file"]
    else:
        config_file = os.path.join(
            get_config_folder(), "config"
        )
    logger.debug("Getting config file %s" % config_file)
    return config_file


def set_config_file(filepath):
    """Override the main configuration file path
    """
    global OVERRIDE_VARS
    if filepath is not None:
        logger.debug("Setting config file to %s" % filepath)
        OVERRIDE_VARS["file"] = filepath


def get_scripts_folder():
    """Get folder where the scripts are stored,
    e.g. /home/user/.papis/scripts
    """
    return os.path.join(
        get_config_folder(), "scripts"
    )


def set(key, val, section=None):
    """Set a key to val in some section and make these changes available
    everywhere.
    >>> set('picktool', 'rofi')
    >>> get('picktool')
    'rofi'
    """
    config = get_configuration()
    if not config.has_section(section or "settings"):
        config.add_section(section or "settings")
    # FIXME: Right now we can only save val in string form
    # FIXME: It would be nice to be able to save also int and booleans
    config.set(section or get_general_settings_name(), key, str(val))


def general_get(key, section=None, data_type=None):
    """General getter method that will be specialised for different modules.

    :param data_type: The data type that should be expected for the value of
        the variable.
    :type  data_type: DataType, e.g. int, src ...
    :param default: Default value for the configuration variable if it is not set.
    :type  default: It should be the same that ``data_type``
    :param extras: List of tuples containing section and prefixes
    """
    # Init main variables
    method = None
    value = None
    config = get_configuration()
    lib = get_lib()
    global_section = get_general_settings_name()
    specialized_key = section + "-" + key if section is not None else key
    extras = [(section, key)] if section is not None else []
    sections = [(global_section, specialized_key)] +\
        extras + [(lib, specialized_key)]
    default_settings = get_default_settings()

    # Check data type for setting getter method
    if data_type == int:
        method = config.getint
    elif data_type == float:
        method = config.getfloat
    elif data_type == bool:
        method = config.getboolean
    else:
        method = config.get

    # Try to get key's value from configuration
    for extra in sections:
        sec = extra[0]
        whole_key = extra[1]
        if sec not in config.keys():
            continue
        if whole_key in config[sec].keys():
            value = method(sec, whole_key)

    if value is None:
        try:
            default = default_settings[
                section or global_section
            ][
                specialized_key if section is None else key
            ]
        except KeyError as e:
            raise papis.exceptions.DefaultSettingValueMissing(key)
        else:
            return default
    return value


def get(*args, **kwargs):
    """String getter
    """
    return general_get(*args, **kwargs)


def getint(*args, **kwargs):
    """Integer getter
    >>> set('something', 42)
    >>> getint('something')
    42
    """
    return general_get(*args, data_type=int, **kwargs)


def getfloat(*args, **kwargs):
    """Float getter
    >>> set('something', 0.42)
    >>> getfloat('something')
    0.42
    """
    return general_get(*args, data_type=float, **kwargs)


def getboolean(*args, **kwargs):
    """Bool getter
    >>> set('add-open', True)
    >>> getboolean('add-open')
    True
    """
    return general_get(*args, data_type=bool, **kwargs)


def in_mode(mode):
    """Get mode of the library. In principle every library can have a mode,
    and depending on the mode some extra options can be offered.

    :param mode: Name of the mode used.
    :type  mode: str
    :returns: Return true if mode matches
    :rtype: bool
    """
    current_mode = get("mode")
    logger.debug("current_mode = %s" % current_mode)
    return mode == current_mode


def get_configuration():
    """Get the configuratoin object, if no papis configuration has ever been
    initialized, it initializes one. Only one configuration per process should
    ever be configurated.

    :returns: Configuration object
    :rtype:  papis.config.Configuration
    """
    global CONFIGURATION
    if CONFIGURATION is None:
        logger.debug("Creating configuration")
        CONFIGURATION = Configuration()
    return CONFIGURATION


def get_lib():
    """Get current library, it either retrieves the library from
    the environment PAPIS_LIB variable or from the command line
    args passed by the user.

    :param library: Name of library or path to a given library
    :type  library: str
    >>> papis.api.set_lib('hello-world')
    >>> get_lib()
    'hello-world'
    """
    import papis.commands
    try:
        lib = papis.commands.get_args().lib
    except AttributeError:
        try:
            lib = os.environ["PAPIS_LIB"]
        except KeyError:
            # Do not put papis.config.get because get is a special function
            # that also needs the library to see if some key was overriden!
            lib = papis.config.get_default_settings(key="default-library")
    return lib


def reset_configuration():
    """Destroys existing configuration and returns a new one.

    :returns: Configuration object
    :rtype:  papis.config.Configuration
    """
    global CONFIGURATION
    if CONFIGURATION is not None:
        logger.warning("Overwriting previous configuration")
    CONFIGURATION = None
    logger.debug("Reseting configuration")
    return get_configuration()


class Configuration(configparser.ConfigParser):

    default_info = {
      "papers": {
        'dir': '~/Documents/papers'
      },
      get_general_settings_name(): {
        'default-library': 'papers'
      }
    }

    logger = logging.getLogger("Configuration")

    def __init__(self):
        configparser.ConfigParser.__init__(self)
        self.dir_location = get_config_folder()
        self.scripts_location = get_scripts_folder()
        self.file_location = get_config_file()
        self.initialize()

    def handle_includes(self):
        if "include" in self.keys():
            for name in self["include"]:
                self.logger.debug("including %s" % name)
                self.read(os.path.expanduser(self.get("include", name)))

    def initialize(self):
        if not os.path.exists(self.dir_location):
            os.makedirs(self.dir_location)
        if not os.path.exists(self.scripts_location):
            os.makedirs(self.scripts_location)
        if os.path.exists(self.file_location):
            self.read(self.file_location)
            self.handle_includes()
        else:
            for section in self.default_info:
                self[section] = {}
                for field in self.default_info[section]:
                    self[section][field] = self.default_info[section][field]
            with open(self.file_location, "w") as configfile:
                self.write(configfile)



import logging

import tempfile
import papis.bibtex
import papis.config

logger = logging.getLogger("downloader")


def getAvailableDownloaders():
    import papis.downloaders.aps
    import papis.downloaders.acs
    import papis.downloaders.arxiv
    import papis.downloaders.scitationaip
    import papis.downloaders.annualreviews
    import papis.downloaders.iopscience
    import papis.downloaders.libgen
    import papis.downloaders.get
    return [
        papis.downloaders.aps.Downloader,
        papis.downloaders.acs.Downloader,
        papis.downloaders.arxiv.Downloader,
        papis.downloaders.scitationaip.Downloader,
        papis.downloaders.annualreviews.Downloader,
        papis.downloaders.iopscience.Downloader,
        papis.downloaders.libgen.Downloader,
        papis.downloaders.get.Downloader,
    ]


def get_downloader(url, downloader=False):
    """Get downloader object. If only a url is given, the url is matched
    against the match method of the downloaders.

    :param url: Url of the document
    :type  url: str
    :param downloader: Name of the downloader, if any.
    :type  downloader: str

    """
    if not downloader:
        for downloader in getAvailableDownloaders():
            result = downloader.match(url)
            if result:
                return result
    else:
        try:
            mod = __import__(
                'papis.downloaders.%s' % downloader,
                fromlist=['Downloader']
            )
            return getattr(mod, 'Downloader')(url)
        except ImportError:
            logger.error("No downloader named %s" % downloader)
    return False


def download_document_from_doi(doi):
    """Download a document knowing the doi number

    :param doi: Doi number
    :type  doi: string
    :returns: Path to downloaded local document

    """
    tempf = tempfile.mktemp()
    external_downloader = papis.config.get(
        "doc-doi-downloader"
    )



def get(url, data_format="bibtex"):
    data = dict()
    documents_paths = []
    doi = None
    logger.debug("Attempting to retrieve from url")
    downloader = get_downloader(url)
    if not downloader:
        logger.warning("Using downloader %s" % downloader)
        return None
    try:
        doi = downloader.getDoi()
    except:
        logger.debug("Doi not found from url...")
    logger.debug("Using downloader %s" % downloader)
    if data_format == "bibtex":
        try:
            bibtex_data = downloader.getBibtexData()
            if bibtex_data:
                data = papis.bibtex.bibtex_to_dict(
                    downloader.getBibtexData()
                )
        except NotImplementedError:
            data = dict()
    try:
        doc_data = downloader.getDocumentData()
    except NotImplementedError:
        doc_data = False
    if doc_data:
        documents_paths.append(tempfile.mktemp())
        logger.debug("Saving in %s" % documents_paths[-1])
        tempfd = open(documents_paths[-1], "wb+")
        tempfd.write(doc_data)
        tempfd.close()
    return {
        "data": data,
        "doi": doi,
        "documents_paths": documents_paths
    }



import papis.commands


class Command(papis.commands.Command):

    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "git",
            help="Run a git command in the library folder"
        )

        self.parser.add_argument(
            "commands",
            help="Commands",
            default="",
            nargs="+",
            action="store"
        )

    def main(self):
        run = papis.commands.get_commands("run")
        run.set_args(self.get_args())
        run.set_commands(["git"] + self.args.commands)
        run.main()



import os
import re
import textwrap
import subprocess
import tempfile
import logging
import papis.config
import papis.commands.external


main_vim_path = os.path.join(
    os.path.dirname(__file__),
    "main.vim"
)

pick_vim_path = os.path.join(
    os.path.dirname(__file__),
    "pick.vim"
)


def pick(
        options,
        header_filter=lambda x: x,
        body_filter=None,
        match_filter=lambda x: x
        ):
    if len(options) == 1:
        return options[0]
    if len(options) == 0:
        return None
    temp_file = tempfile.mktemp()
    fd = open(temp_file, "w+")
    headers = [
        header_filter(d) for d in
        options
    ]
    for header in headers:
        fd.write(header+"\n")
    fd.write("# Put your cursor on a line and press enter to pick\n")
    fd.close()
    process = subprocess.call(
        ["vim", "-S", pick_vim_path, temp_file]
    )
    fd = open(temp_file)
    index = fd.read()
    fd.close()
    ret = options[int(index)-1]
    return ret


class Gui(object):


    def __init__(self):
        self.documents = []
        self.logger = logging.getLogger("gui:vim")
        self.main_vim_path = main_vim_path


    def export_variables(self, args):
        """Export variables so that vim can use some papis information
        """
        external_cmd = papis.commands.external.Command()
        external_cmd.set_args(args)
        external_cmd.export_variables()

    def main(self, documents, args):
        header_format = papis.config.get(
            "header-format",
            section="vim-gui"
        )
        self.export_variables(args)
        temp_file = tempfile.mktemp()
        self.logger.debug("Temp file = %s" % temp_file)
        fd = open(temp_file, "w+")
        for doc in documents:
            fd.write(
                papis.utils.format_doc(header_format, doc)
            )
        fd.close()

        subprocess.call(
                ["vim", "-S", self.main_vim_path, temp_file]
        )



import papis.utils
import papis.config


class Command(papis.commands.Command):
    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "gui",
            help="Graphical user interface"
        )

        self.parser.add_argument(
            "--tk",
            help="Tk based UI",
            action="store_true"
        )

        self.parser.add_argument(
            "--rofi",
            help="Rofi based UI",
            action="store_true"
        )

        self.parser.add_argument(
            "--vim",
            help="Vim based UI",
            action="store_true"
        )

    def fetch_documents(self):
        return papis.api.get_documents_in_lib(self.args.lib)

    def rofi_main(self):
        import papis.gui.rofi
        return papis.gui.rofi.Gui().main(self.documents)

    def tk_main(self):
        import papis.gui.tk
        return papis.gui.tk.Gui().main(self.documents)

    def vim_main(self):
        import papis.gui.vim
        return papis.gui.vim.Gui().main(self.documents, self.args)

    def main(self):
        self.documents = self.fetch_documents()
        if self.args.tk:
            return self.tk_main()
        if self.args.vim:
            return self.vim_main()
        if self.args.rofi:
            return self.rofi_main()
        default = papis.config.get("default-gui")
        return exec("self.%s_main()" % default)




import papis.config
import papis.utils
import sys
import logging
from papis.gui.tk import tk
import papis.gui.tk


class PapisWidget(tk.Misc):

    normal_mode = "normal"
    insert_mode = "insert"
    command_mode = "command"

    CURRENT_MODE = normal_mode

    def __init__(self):
        tk.Misc.__init__(self)
        self.logger = logging.getLogger(self.__class__.__name__)

    def get_mode(self):
        return PapisWidget.CURRENT_MODE

    def set_mode(self, mode):
        self.logger.debug("Mode -> %s" % mode)
        PapisWidget.CURRENT_MODE = mode

    def general_map(self, key, function, mode=None, recursive=False):
        def help_function(*args, **kwargs):
            if self.get_mode() == mode or mode is None:
                return function(*args, **kwargs)
        if recursive:
            self.bind_all(key, help_function)
        else:
            self.bind(key, help_function)

    def noremap(self, key, function, mode=None):
        self.general_map(key, function, mode, recursive=True)

    def norenmap(self, key, function):
        self.noremap(key, function, self.normal_mode)

    def noreimap(self, key, function):
        self.noremap(key, function, self.insert_mode)

    def norecmap(self, key, function):
        self.noremap(key, function, self.command_mode)

    def map(self, key, function, mode=None):
        self.general_map(key, function, mode, recursive=False)

    def nmap(self, key, function):
        self.map(key, function, self.normal_mode)

    def imap(self, key, function):
        self.map(key, function, self.insert_mode)

    def cmap(self, key, function):
        self.map(key, function, self.command_mode)

    def to_normal(self, event=None):
        self.focus()
        self.set_mode(self.normal_mode)

    def get_config(self, key):
        """Get configuration to key
        """
        return papis.config.get(
            key, section="tk-gui"
        )


class Gui(tk.Tk, PapisWidget):

    def __init__(self):
        tk.Tk.__init__(self)
        PapisWidget.__init__(self)
        self.protocol("WM_DELETE_WINDOW", self.exit)
        self.geometry(
            "{}x{}".format(
                self.get_config("window-width"),
                self.get_config("window-height"),
            )
        )
        self["bg"] = self.get_config("window-bg")
        self.title("Papis document manager")
        self.prompt = papis.gui.tk.Prompt(
            self,
        )
        self.main_frame = papis.gui.tk.PapisList(
            self
        )
        self.prompt.add_new_command("help", self.print_help)
        self.prompt.add_new_command("q", self.exit)
        self.prompt.add_new_command("quit", self.exit)

    def set_bindings(self):
        self.logger.debug("Setting bindings")
        self.norecmap("<Return>", self.prompt.run)
        self.noremap("<Escape>", self.clear)
        self.cmap("<Control-c>", self.to_normal)
        self.map("<Configure>", self.on_resize)
        self.bindings = [
            (self.get_config("exit"), "exit"),
            (self.get_config("focus_prompt"), "focus_prompt"),
            (self.get_config("clear"), "clear"),
            (self.get_config("help"), "print_help"),
        ]
        for bind in self.bindings:
            key = bind[0]
            name = bind[1]
            self.nmap(key, getattr(self, name))

    def on_resize(self, event=None):
        pass

    def exit(self, event=None):
        self.logger.debug("Exiting")
        self.destroy()
        sys.exit(0)

    def clear(self, event=None):
        self.prompt.clear()
        self.to_normal()

    def handle_return(self, event=None):
        self.prompt.get_command()
        self.prompt.clear()
        self.focus()

    def focus_prompt(self, event=None):
        self.prompt.clear()
        self.prompt.focus()

    def main(self, documents):
        self.set_bindings()
        self.logger.debug("Packing prompt")
        self.prompt.pack(fill=tk.X, side=tk.BOTTOM)
        self.logger.debug("Setting docs")
        # self.main_frame.pack(fill=tk.X, side=tk.TOP)
        self.main_frame.set_documents(documents)
        self.after(
            200,
            self.main_frame.init
        )
        return self.mainloop()

    def print_help(self, event=None):
        text = ""
        for bind in self.bindings + self.main_frame.bindings:
            text += "{key}  -  {name}\n".format(key=bind[0], name=bind[1])
        self.prompt.echomsg(text)



import papis
import os
import sys
import papis.api
import papis.utils


class Command(papis.commands.Command):
    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "open",
            help="Open document document from a given library"
        )

        self.add_search_argument()

        self.parser.add_argument(
            "--tool",
            help="Tool for opening the file (opentool)",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "-d",
            "--dir",
            help="Open directory",
            action="store_true"
        )

    def main(self):
        if self.args.tool:
            papis.config.set("opentool", self.args.tool)

        documents = papis.api.get_documents_in_lib(
            self.get_args().lib,
            self.args.search
        )
        if not documents:
            print("No documents found with that name.")
            sys.exit(1)

        document = self.pick(documents)
        if not document:
            sys.exit(0)

        if not self.args.dir:
            files = document.get_files()
            file_to_open = papis.api.pick(
                files,
                pick_config=dict(
                    header_filter=lambda x: x.replace(
                        document.get_main_folder(), ""
                    )
                )
            )
            papis.api.open_file(file_to_open)
        else:
            # Open directory
            papis.api.open_dir(document.get_main_folder())



import papis
import sys
import os
import re
import papis.api
import papis.utils
import subprocess


class Command(papis.commands.Command):
    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "rename",
            help="Rename entry"
        )

        self.add_search_argument()
        self.add_git_argument()


    def main(self):

        documents = papis.api.get_documents_in_lib(
            self.get_args().lib,
            self.get_args().search
        )

        document = self.pick(documents) or sys.exit(0)

        folder = document.get_main_folder()
        subfolder = os.path.dirname(folder)

        new_folder = os.path.join(
            subfolder,
            papis.utils.clean_document_name(
                papis.utils.input(
                    "Enter new folder name:\n"
                    ">",
                    default=document.get_main_folder_name()
                )
            )
        )

        self.logger.info(new_folder)

        if os.path.exists(new_folder):
            self.logger.exception("Path %s already exists!" % new_folder)

        mvtool = papis.config.get("mvtool")

        cmd = (['git', '-C', folder] if self.args.git else []) + \
            ['mv', folder, new_folder]
        self.logger.debug(cmd)
        subprocess.call(cmd)
        papis.utils.git_commit(message="Rename %s" % folder)
        papis.api.clear_lib_cache()



import rofi
import webbrowser
import papis.api
import papis.utils
import papis.config


def get_options():
    options = dict()

    for key in ["fullscreen",
            "normal_window", "multi_select", "case_sensitive", "markup_rows"]:
        options[key] =\
            papis.config.getboolean(key, section="rofi-gui")

    for key in ["width", "eh", "lines", "fixed_lines"]:
        options[key] =\
            papis.config.getint(key, section="rofi-gui")

    for key in ["sep"]:
        options[key] =\
            papis.config.get(key, section="rofi-gui")

    return options


def pick(options, header_filter=None, body_filter=None, match_filter=None):
    if header_filter is None:
        header_filter = lambda x: papis.utils.format_doc(
            papis.config.get('header-format', section='rofi-gui'), x
        )
    if len(options) == 1:
        indices = 0
    else:
        r = rofi.Rofi()
        indices, key = r.select(
            "Select: ",
            [
                header_filter(d) for d in
                options
            ],
            **get_options()
        )
        r.close()
    # TODO: Support multiple choice
    if not isinstance(indices, list):
        indices = [indices]
    return options[indices[0]]


class Gui(object):

    esc_key = -1
    open_key = 0
    quit_key = 1
    edit_key = 2
    delete_key = 3
    help_key = 4
    open_stay_key = 5
    normal_widnow_key = 6
    browse_key = 7

    def __init__(self):
        self.documents = []
        self.help_message = ""
        self.keys = self.get_keys()
        self.window = None

    def get_help(self):
        space = " "*10
        message = \
"Rofi based gui for papis\n"\
"========================\n".format(space)
        for k in self.keys:
            message += "%s%s%s\n" % (self.keys[k][0], space, self.keys[k][1])
        return message

    def get_key(self, key):
        return papis.config.get("key-" + key.lower(), section="rofi-gui")

    def get_keys(self):
        return {
            "key%s" % self.quit_key: (
                self.get_key('quit'),
                'Quit'
            ),
            "key%s" % self.edit_key: (
                self.get_key('edit'),
                'Edit'
            ),
            "key%s" % self.delete_key: (
                self.get_key('delete'),
                'Delete'
            ),
            "key%s" % self.help_key: (
                self.get_key('help'),
                'Help'
            ),
            "key%s" % self.open_stay_key: (
                self.get_key('open-stay'),
                'Open'
            ),
            "key%s" % self.normal_widnow_key: (
                self.get_key('normal-window'),
                'Normal Win'
            ),
            "key%s" % self.open_key: (
                self.get_key('open'),
                'Open'
            ),
            "key%s" % self.browse_key: (
                self.get_key('browse'),
                'Browse'
            )
        }

    def main(self, documents):
        # Set default picker
        self.documents = documents
        key = None
        indices = None
        options = get_options()
        header_format = papis.config.get("header-format", section="rofi-gui")
        header_filter = lambda x: papis.utils.format_doc(header_format, x)
        self.help_message = self.get_help()
        options.update(self.keys)
        # Initialize window
        self.window = rofi.Rofi()
        while not (key == self.quit_key or key == self.esc_key):
            indices, key = self.window.select("Select: ",
                [
                    header_filter(d) for d in
                    self.documents
                ],
                select=indices,
                **options
            )
            if not isinstance(indices, list):
                indices = [indices]
            if key == self.edit_key:
                for i in indices:
                    self.edit(self.documents[i])
            elif key in [self.open_key, self.open_stay_key]:
                for i in indices:
                    self.open(self.documents[i])
                options["normal_window"] ^= True
                if key == self.open_key:
                    return 0
            elif key == self.delete_key:
                for i in indices:
                    self.delete(self.documents[i])
            elif key == self.help_key:
                self.window.error(self.help_message)
            elif key == self.browse_key:
                for i in indices:
                    self.browse(self.documents[i])
            elif key == self.normal_widnow_key:
                options["normal_window"] ^= True

    def delete(self, doc):
        answer = self.window.text_entry(
            "Are you sure? (y/N)",
            message="<b>Be careful!</b>"
        )
        if answer and answer in "Yy":
            doc.rm()
            self.documents = self.fetch_documents()

    def open(self, doc):
        papis.api.open_file(
            doc.get_files()
        )

    def browse(self, doc):
        url = doc["url"]
        if not url:
            self.window.error("No url for document found")
        else:
            papis.utils.general_open(
                doc["url"], "browser"
            )

    def edit(self, doc):
        papis.utils.general_open(
            doc.get_info_file(),
            "xeditor",
            wait=True
        )
        doc.load()



# -*- coding: utf-8 -*-
#
# you can install this to a local test virtualenv like so:
#   virtualenv venv
#   ./venv/bin/pip install --editable .
#   ./venv/bin/pip install --editable .[dev]  # with dev requirements, too

import sys

main_dependencies = [
    "setuptools",
    "curses"
]

for dep in main_dependencies:
    try:
        __import__(dep)
    except ImportError:
        print(
            "Error: You do not have %s installed, please\n"
            "       install it. For example doing\n"
            "\n"
            "       pip3 install %s\n" % (dep, dep)
        )
        sys.exit(1)


from setuptools import setup
import papis


setup(
    name='papis',
    version=papis.__version__,
    maintainer='Alejandro Gallo',
    maintainer_email='aamsgallo@gmail.com',
    author=papis.__author__,
    author_email=papis.__email__,
    license=papis.__license__,
    url='https://github.com/alejandrogallo/papis',
    install_requires=[
        "papis-python-rofi>=1.0.1",
        "requests>=2.11.1",
        "argcomplete>=1.8.2",
        "configparser>=3.0.0",
        "arxiv2bib>=1.0.7",
        "PyYAML>=3.12",
        "chardet>=3.0.2",
        "beautifulsoup4>=4.4.1",
        "vobject>=0.9.4.1",
        "prompt-toolkit>=1.0.0",
    ],
    python_requires='>=3',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Environment :: Console :: Curses',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Utilities',
    ],
    extras_require=dict(
        # List additional groups of dependencies here (e.g. development
        # dependencies). You can install these using the following syntax,
        # for example:
        # $ pip install -e .[develop]
        develop=[
            "sphinx",
            'sphinx-argparse',
            'sphinx_rtd_theme',
            'pytest',
        ]
    ),
    description='Simple program to manage literature',
    long_description='Simple program to manage literature',
    keywords=[
        'document',
        'books',
        'bibtex',
        'management',
        'cli',
        'biliography'
    ],
    package_data=dict(
        papis=[
            'gui/vim/main.vim',
        ],
    ),
    data_files=[

        ("share/doc/papis/", [
            "README.md",
        ]),

        ("share/man/man1", [
            "doc/build/man/papis.1",
            "doc/build/man/papis-config.1",
        ]),

        ("share/applications", [
            "contrib/papis.desktop",
        ]),

    ],
    packages=[
        "papis",
        "papis.gui",
        "papis.gui.tk",
        "papis.gui.vim",
        "papis.commands",
        "papis.downloaders",
    ],
    test_suite="papis.tests",
    entry_points=dict(
        console_scripts=[
            'papis=papis.main:main'
        ]
    ),
    platforms=['linux', 'osx'],
)



import os
import string
import papis.commands
import papis.config


class Command(papis.commands.Command):

    def init(self):
        self.parser = self.get_subparsers().add_parser(
            "sync",
            help="Sync a library using the sync command"
        )

    def main(self):
        sync_command = os.path.expanduser(
            papis.config.get("sync-command")
        )
        command = string.Template(sync_command).substitute(
            self.get_config()[self.args.lib]
        )
        print(command)
        os.system(command)



import re
import logging
import os

logger = logging.getLogger("bibtex")

bibtex_types = [
  "article",
  "book",
  "booklet",
  "conference",
  "inbook",
  "incollection",
  "inproceedings",
  "manual",
  "mastersthesis",
  "misc",
  "phdthesis",
  "proceedings",
  "techreport",
  "unpublished"
]

bibtex_keys = [
  "address",
  "annote",
  "author",
  "booktitle",
  "doi",
  "chapter",
  "crossref",
  "edition",
  "editor",
  "howpublished",
  "institution",
  "journal",
  "key",
  "month",
  "note",
  "number",
  "organization",
  "pages",
  "publisher",
  "school",
  "series",
  "title",
  "volume",
  "year"
  ]


def bibtex_to_dict(bibtex):
    """
    Convert bibtex file to dict

    .. code:: python

        { type: "article ...", "ref": "example1960etAl", author:" ..."}

    :param bibtex: Bibtex file path or bibtex information in string format.
    :type  bibtex: str
    :returns: Dictionary with bibtex information with keys that bibtex
        formally recognizes.
    :rtype:  dict
    """
    global logger
    result = dict()
    if os.path.exists(bibtex):
        with open(bibtex) as fd:
            logger.debug("Reading in file %s" % bibtex)
            text = fd.read()
    else:
        text = bibtex
    logger.debug("Removing comments...")
    text = re.sub(r"%.*", "", text)
    logger.debug("Removing empty lines...")
    text = re.sub(r"^\s*$", "", text)
    logger.debug("Removing newlines...")
    text = re.sub(r"\n", "", text)
    logger.debug("Parsing document type and reference")
    type_ref_re = re.compile(r"\s*@\s*(\w+)\s*{([^,]+),?")
    match = re.match(type_ref_re, text)
    text = re.sub(type_ref_re, "", text)
    if not match:
        logger.error(
            "Type and reference of the bibtex file could not be parsed",
            exc_info=False
        )
        return result
    result["type"] = match.group(1)
    result["ref"] = match.group(2)
    for key in result.keys():
        logger.debug(" [%s] = %s" % (key, result[key]))
    key_val_re = re.compile(r"\s*(\w+)\s*=\s*{(.*)\s*,?")
    for line in re.sub(r"}\s*,?", "\n", text).split("\n"):
        match = re.match(key_val_re, line)
        if match:
            key = match.group(1)
            val = re.sub(r"\s+", " ", match.group(2))
            result[key.lower()] = val
            logger.debug("[%s] = %s" % (key, val))
    return result



import string
import os
import papis.config
import papis.exceptions


class Command(papis.commands.Command):

    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "run",
            help="Run a command in the library folder"
        )

        self.parser.add_argument(
            "run_command",
            help="Command name or command",
            default="",
            nargs="+",
            action="store"
        )

    def set_commands(self, commands):
        """Set commands to be run.
        :param commands: List of commands
        :type  commands: list
        """
        self.args.run_command = commands

    def main(self):
        lib_dir = os.path.expanduser(self.get_config()[self.args.lib]["dir"])
        self.logger.debug("Changing directory into %s" % lib_dir)
        os.chdir(lib_dir)
        try:
            command = os.path.expanduser(
                papis.config.get("".join(self.args.run_command))
            )
        except papis.exceptions.DefaultSettingValueMissing:
            command = " ".join(self.args.run_command)
        self.logger.debug("Command = %s" % command)
        command = string.Template(command).safe_substitute(
            self.get_config()[self.args.lib]
        )
        os.system(command)
